@@ -187,7 +187,6 @@ class Buildinfo:
                 self.urls[node.get('project') + "/" + node.get('repository')] = baseurl + '/%(arch)s/%(filename)s'
 
         self.vminstall_list = [dep.name for dep in self.deps if dep.vminstall]
-        self.sb2install_list = [dep.name for dep in self.deps if dep.sb2install]
         self.preinstall_list = [dep.name for dep in self.deps if dep.preinstall]
         self.runscripts_list = [dep.name for dep in self.deps if dep.runscripts]
         self.noinstall_list = [dep.name for dep in self.deps if dep.noinstall]
@@ -235,8 +234,7 @@ class Pac:
                   'project', 'repository', 'sysroot',
                   'preinstall', 'vminstall', 'runscripts',
                   'noinstall', 'installonly', 'notmeta',
-                  'sb2install',
-                 ]:
+                  ]:
             self.mp[i] = node.get(i)
 
         self.mp['buildarch'] = buildarch
@@ -822,13 +820,6 @@ def main(apiurl, store, opts, argv):
         buildargs.append('--clean')
     if opts.checks:
         buildargs.append('--checks')
-    if opts.rebuild:
-        if not opts.rsyncsrc or not opts.rsyncdest:
-            print('Warning: --rebuild option is usually used with both --rsync-src and --rsync-dest',
-                  file=sys.stderr)
-        buildargs.append('--skip-prep')
-    if opts.chroot_only:
-        buildargs.append('--chroot-only')
     if opts.nochecks:
         buildargs.append('--no-checks')
     if not opts.no_changelog:
@@ -950,7 +941,10 @@ def main(apiurl, store, opts, argv):
     bc_file = None
     bi_filename = '_buildinfo-%s-%s.xml' % (repo, arch)
     bc_filename = '_buildconfig-%s-%s' % (repo, arch)
-    if store is not None and store.is_package and os.access(core.store, os.W_OK):
+    if store is not None and hasattr(store, "cache_get_path"):
+        bi_filename = store.cache_get_path(bi_filename, makedirs=True)
+        bc_filename = store.cache_get_path(bc_filename, makedirs=True)
+    elif store is not None and store.is_package and os.access(core.store, os.W_OK):
         bi_filename = os.path.join(os.getcwd(), core.store, bi_filename)
         bc_filename = os.path.join(os.getcwd(), core.store, bc_filename)
     elif not os.access('.', os.W_OK):
@@ -1507,7 +1501,6 @@ def main(apiurl, store, opts, argv):
 
     rpmlist.append('preinstall: ' + ' '.join(bi.preinstall_list) + '\n')
     rpmlist.append('vminstall: ' + ' '.join(bi.vminstall_list) + '\n')
-    rpmlist.append('sb2install: ' + ' '.join(bi.sb2install_list) + '\n')
     rpmlist.append('runscripts: ' + ' '.join(bi.runscripts_list) + '\n')
     if build_type != 'kiwi' and build_type != 'docker' and build_type != 'podman' and build_type != 'fissile':
         if bi.noinstall_list:
